package com.install4j.api.windows.service;

import java.io.Serializable;

/**
 * The service configuration passed to {@link WinServices}. If you don't set a property the original value will be preserved for existing services.
 * @author ej-technologies GmbH
 */
public class ServiceConfiguration implements Serializable {
    private String binaryName;
    private String displayName;
    private ServiceStartType startType;
    private String dependencies;
    private String description;
    private Boolean delayedAutoStart;
    private Boolean restartOnFailure;

    private ServiceAccount serviceAccount;
    private String otherAccountName;
    private String otherAccountPassword;

    /**
     * The path to the binary. This can be quoted and contain arguments
     * @param binaryName the binary name
     * @return this object
     */
    public ServiceConfiguration binaryName(String binaryName) {
        this.binaryName = binaryName;
        return this;
    }

    /**
     * The display name. This is different from the service name which serves as identifier.
     * @param displayName the display name
     * @return this object
     */
    public ServiceConfiguration displayName(String displayName) {
        this.displayName = displayName;
        return this;
    }

    /**
     * The start type
     * @param startType the start type
     * @return this object
     */
    public ServiceConfiguration startType(ServiceStartType startType) {
        this.startType = startType;
        return this;
    }

    /**
     * A comma separated list of dependencies
     * @param dependencies the dependencies
     * @return this object
     */
    public ServiceConfiguration dependencies(String dependencies) {
        this.dependencies = dependencies;
        return this;
    }

    /**
     * An additional description for the service
     * @param description the description
     * @return this object
     */
    public ServiceConfiguration description(String description) {
        this.description = description;
        return this;
    }

    /**
     * If startType is set to <tt>AUTO</tt>, you can specify with this property that the service is started with a short delay after all other auto-start services.
     * @param delayedAutoStart <tt>true</tt> if the service start should be delayed
     * @return this object
     */
    public ServiceConfiguration delayedAutoStart(Boolean delayedAutoStart) {
        this.delayedAutoStart = delayedAutoStart;
        return this;
    }

    /**
     * If set to true, the service will be restarted after a second if it was terminated with an error
     * @param restartOnFailure <tt>true</tt> if a restart should be performed
     * @return this object
     */
    public ServiceConfiguration restartOnFailure(Boolean restartOnFailure) {
        this.restartOnFailure = restartOnFailure;
        return this;
    }

    /**
     * The service account to be used. Can be one of the predefined Windows account or <tt>OTHER</tt>. If it is <tt>OTHER</tt>, you have to specify
     * otherAccountName and otherAccountPassword
     * @param serviceAccount the account to be used
     * @return this object
     */
    public ServiceConfiguration serviceAccount(ServiceAccount serviceAccount) {
        this.serviceAccount = serviceAccount;
        return this;
    }

    /**
     * The service account name to be used. The serviceAccount property will be set to <tt>OTHER</tt> when you call this method.
     * The name must be in the form DomainName\UserName. If the account belongs to the built-in domain, you can specify .\UserName.
     * @param otherAccountName the account name or SID
     * @return this object
     */
    public ServiceConfiguration otherAccountName(String otherAccountName) {
        serviceAccount = ServiceAccount.OTHER;
        this.otherAccountName = otherAccountName;
        return this;
    }

    /**
     * The service account password to be used. The serviceAccount property will be set to <tt>OTHER</tt> when you call this method.
     * @param otherAccountPassword the account password
     * @return this object
     */
    public ServiceConfiguration otherAccountPassword(String otherAccountPassword) {
        serviceAccount = ServiceAccount.OTHER;
        this.otherAccountPassword = otherAccountPassword;
        return this;
    }

    String getBinaryName() {
        return binaryName;
    }

    String getDisplayName() {
        return displayName;
    }

    ServiceStartType getStartType() {
        return startType;
    }

    String getDependencies() {
        return dependencies;
    }

    ServiceAccount getServiceAccount() {
        return serviceAccount;
    }

    String getOtherAccountName() {
        return otherAccountName;
    }

    String getOtherAccountPassword() {
        return otherAccountPassword;
    }

    String getDescription() {
        return description;
    }

    Boolean getDelayedAutoStart() {
        return delayedAutoStart;
    }

    Boolean getRestartOnFailure() {
        return restartOnFailure;
    }
}
